from base.html_email import html_email
from django.conf import settings
from django.core.mail import send_mail
from django.utils.html import escape
from django.utils.translation import gettext as _


def send_submit_notification(
    document_title,
    link,
    message,
    editor_name,
    editor_email,
):

    if len(document_title) == 0:
        document_title = _("Untitled")
    message_text = _(
        f"Hey {editor_name}, the document '{document_title}' has "
        "been submitted to be published. You or another editor need to "
        "approve it before it will be made accessible to the general "
        "public."
        f"\nOpen the document: {link}",
    )
    body_html_intro = _(
        f"<p>Hey {escape(editor_name)}<br>the document '{escape(document_title)}' has "
        "been submitted to be published. You or another editor need to "
        "approve it before it will be made accessible to the general "
        "public.</p>",
    )
    if len(message):
        message_text += _(f"\nMessage from the author: {message}")
        body_html_intro += _(
            f"<p>Message from the author: {escape(message)}</p>",
        )
    review_document_str = _(f"Review {escape(document_title)}")
    access_the_document_str = _("Access the Document")
    document_str = _("Document")
    body_html = (
        f"<h1>{review_document_str}</h1>"
        f"{body_html_intro}"
        "<table>"
        f"<tr><td>{document_str}</td><td>"
        f"<b>{document_title}</b>"
        "</td></tr>"
        "</table>"
        f'<div class="actions"><a class="button" href="{link}">'
        f"{access_the_document_str}"
        "</a></div>"
    )
    send_mail(
        _(f"Document shared: {escape(document_title)}"),
        message_text,
        settings.DEFAULT_FROM_EMAIL,
        [editor_email],
        fail_silently=True,
        html_message=html_email(body_html),
    )


def send_review_notification(
    document_title,
    link,
    message,
    author_name,
    author_email,
):

    if len(document_title) == 0:
        document_title = _("Untitled")
    message_text = _(
        f"Hey {author_name}, the document '{document_title}' has "
        "been reviewed. You or another author need to change some things "
        "before it can be published. Please resubmit it once you are done. "
        f"\nOpen the document: {link}",
    )
    body_html_intro = _(
        f"<p>Hey {escape(author_name)}<br>the document '{escape(document_title)}' has "
        "been reviewed. You or another author need to change some things "
        "before it can be published. Please resubmit it once you are done.</p>",
    )
    if len(message):
        message_text += _(f"\nMessage from the editor: {message}")
        body_html_intro += _(
            f"<p>Message from the editor: {escape(message)}</p>",
        )
    review_document_str = _(f"{escape(document_title)} reviewed")
    access_the_document_str = _("Access the Document")
    document_str = _("Document")
    body_html = (
        f"<h1>{review_document_str}</h1>"
        f"{body_html_intro}"
        "<table>"
        f"<tr><td>{document_str}</td><td>"
        f"<b>{document_title}</b>"
        "</td></tr>"
        "</table>"
        f'<div class="actions"><a class="button" href="{link}">'
        f"{access_the_document_str}"
        "</a></div>"
    )
    send_mail(
        _(f"Document: {escape(document_title)}"),
        message_text,
        settings.DEFAULT_FROM_EMAIL,
        [author_email],
        fail_silently=True,
        html_message=html_email(body_html),
    )


def send_reject_notification(
    document_title,
    link,
    message,
    author_name,
    author_email,
):

    if len(document_title) == 0:
        document_title = _("Untitled")
    message_text = _(
        f"Hey {author_name}, the document '{document_title}' has "
        "been reviewed and rejected. "
        f"\nOpen the document: {link}",
    )
    body_html_intro = _(
        f"<p>Hey {escape(author_name)}<br>the document '{escape(document_title)}' has "
        "been reviewed and rejected.</p>",
    )
    if len(message):
        message_text += _(f"\nMessage from the editor: {message}")
        body_html_intro += _(
            f"<p>Message from the editor: {escape(message)}</p>",
        )
    review_document_str = _(f"{escape(document_title)} rejected")
    access_the_document_str = _("Access the Document")
    document_str = _("Document")
    body_html = (
        f"<h1>{review_document_str}</h1>"
        f"{body_html_intro}"
        "<table>"
        f"<tr><td>{document_str}</td><td>"
        f"<b>{document_title}</b>"
        "</td></tr>"
        "</table>"
        f'<div class="actions"><a class="button" href="{link}">'
        f"{access_the_document_str}"
        "</a></div>"
    )
    send_mail(
        _(f"Document: {escape(document_title)}"),
        message_text,
        settings.DEFAULT_FROM_EMAIL,
        [author_email],
        fail_silently=True,
        html_message=html_email(body_html),
    )


def send_publish_notification(
    document_title,
    link,
    message,
    author_name,
    author_email,
):

    if len(document_title) == 0:
        document_title = _("Untitled")
    message_text = _(
        f"Hey {author_name}, the document '{document_title}' has "
        "been reviewed and published. "
        f"\nOpen the document: {link}",
    )
    body_html_intro = _(
        f"<p>Hey {escape(author_name)}<br>the document '{escape(document_title)}' has "
        "been reviewed and published.</p>",
    )
    if len(message):
        message_text += _(f"\nMessage from the editor: {message}")
        body_html_intro += _(
            f"<p>Message from the editor: {escape(message)}</p>",
        )
    review_document_str = _(f"{escape(document_title)} published")
    access_the_document_str = _("Access the Document")
    document_str = _("Document")
    body_html = (
        f"<h1>{review_document_str}</h1>"
        f"{body_html_intro}"
        "<table>"
        f"<tr><td>{document_str}</td><td>"
        f"<b>{document_title}</b>"
        "</td></tr>"
        "</table>"
        f'<div class="actions"><a class="button" href="{link}">'
        f"{access_the_document_str}"
        "</a></div>"
    )
    send_mail(
        _(f"Document: {escape(document_title)}"),
        message_text,
        settings.DEFAULT_FROM_EMAIL,
        [author_email],
        fail_silently=True,
        html_message=html_email(body_html),
    )



# Generated by Django 4.0.4 on 2022-05-13 20:03

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0008_auto_20210604_0758"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="contacts",
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL),
        ),
    ]



from django.contrib import admin

from . import models


class PublicationAdmin(admin.ModelAdmin):
    pass


admin.site.register(models.Publication, PublicationAdmin)


class PublicationAssetAdmin(admin.ModelAdmin):
    pass


admin.site.register(models.PublicationAsset, PublicationAssetAdmin)


class EditorAdmin(admin.ModelAdmin):
    pass


admin.site.register(models.Editor, EditorAdmin)


class DesignAdmin(admin.ModelAdmin):
    pass


admin.site.register(models.Design, DesignAdmin)



# Generated by Django 2.2.9 on 2020-02-19 13:52

from django.db import migrations, models
import django.db.models.deletion
import style.models


class Migration(migrations.Migration):

    replaces = [
        ("style", "0001_squashed_0002_auto_20151226_1110"),
        ("style", "0002_auto_20190316_0241"),
        ("style", "0003_auto_20190803_0749"),
        ("style", "0004_auto_20190809_1655"),
        ("style", "0005_documentstyle_documentstylefile_exporttemplate"),
        ("style", "0006_auto_20190809_1757"),
        ("style", "0007_auto_20190811_1204"),
        ("style", "0008_auto_20190830_0627"),
    ]

    initial = True

    dependencies = [
        ("document", "0001_squashed_20200219"),
    ]

    operations = [
        migrations.CreateModel(
            name="DocumentStyle",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="Default",
                        help_text="The human readable title.",
                        max_length=128,
                    ),
                ),
                (
                    "slug",
                    models.SlugField(
                        default="default",
                        help_text="The base of the filenames the style occupies.",
                        max_length=20,
                    ),
                ),
                (
                    "contents",
                    models.TextField(
                        default="", help_text="The CSS style definiton."
                    ),
                ),
                (
                    "document_template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="document.DocumentTemplate",
                    ),
                ),
            ],
            options={
                "unique_together": {("slug", "document_template")},
            },
        ),
        migrations.CreateModel(
            name="ExportTemplate",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        default="Default",
                        help_text="The human readable title.",
                        max_length=128,
                    ),
                ),
                (
                    "file_type",
                    models.CharField(
                        choices=[("docx", "DOCX"), ("odt", "ODT")],
                        max_length=5,
                    ),
                ),
                (
                    "template_file",
                    models.FileField(upload_to=style.models.template_filename),
                ),
                (
                    "document_template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="document.DocumentTemplate",
                    ),
                ),
            ],
            options={
                "unique_together": {("title", "document_template")},
            },
        ),
        migrations.CreateModel(
            name="DocumentStyleFile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "file",
                    models.FileField(
                        help_text="A file references in the style. The filename will be replaced with the final url of the file in the style.",
                        upload_to=style.models.documentstylefile_location,
                    ),
                ),
                (
                    "filename",
                    models.CharField(
                        help_text="The original filename.", max_length=255
                    ),
                ),
                (
                    "style",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="style.DocumentStyle",
                    ),
                ),
            ],
            options={
                "unique_together": {("filename", "style")},
            },
        ),
    ]



import time
from random import randrange
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from testing.selenium_helper import SeleniumHelper
from document.models import Document


class EditorHelper(SeleniumHelper):
    """
    Common functions used in threaded tests
    """

    def create_new_document(self):
        doc = Document.objects.create(
            owner=self.user, template_id=1
        )  # from fixture
        doc.save()
        return doc

    def load_document_editor(self, driver, doc):
        driver.get("%s%s" % (self.live_server_url, doc.get_absolute_url()))
        WebDriverWait(driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, "editor-toolbar"))
        )
        self.inject_helpers(driver)

    def inject_helpers(self, driver):
        with open("static-transpile/js/test_caret.js", "r") as file:
            test_caret_script = file.read()
        driver.execute_script(test_caret_script)

    def input_text(self, document_input, text):
        for char in text:
            document_input.send_keys(char)
            time.sleep(randrange(10, 40) / 200.0)

    def type_text(self, driver, text):
        for char in text:
            actions = ActionChains(driver)
            actions.send_keys(char)
            actions.perform()
            time.sleep(randrange(10, 40) / 200.0)

    def add_title(self, driver):
        title = "My title"
        self.driver.find_element(By.CLASS_NAME, "article-title").send_keys(
            title
        )

    def get_contents(self, driver):
        # Contents is child 5.
        return driver.execute_script(
            "return window.theApp.page.view.state.doc.firstChild"
            ".child(5).textContent;"
        )

    def wait_for_doc_size(self, driver, size, seconds=False):
        if seconds is False:
            seconds = self.wait_time
        doc_size = driver.execute_script(
            "return window.theApp.page.view.state.doc.content.size"
        )
        if doc_size < size and seconds > 0:
            time.sleep(0.1)
            self.wait_for_doc_size(driver, size, seconds - 0.1)

    def wait_for_doc_sync(self, driver, driver2, seconds=False):
        # Wait at least 1/4 second just in case documents are about to change.
        time.sleep(0.25)
        if seconds is False:
            seconds = self.wait_time
        doc_str = driver.execute_script(
            "return window.theApp.page.view.state.doc.toString()"
        )
        doc2_str = driver2.execute_script(
            "return window.theApp.page.view.state.doc.toString()"
        )
        if doc_str != doc2_str:
            # The strings don't match.
            time.sleep(0.1)
            self.wait_for_doc_sync(driver, driver2, seconds - 0.1)

    def add_footnote(
        self, driver, footnote_pos, footnote_content, footnote_num
    ):
        driver.execute_script(
            f"window.testCaret.setSelection({footnote_pos},{footnote_pos})"
        )
        driver.find_element(By.XPATH, '//*[@title="Footnote"]').click()

        driver.find_element(
            By.XPATH,
            f'//*[@id="footnote-box-container"]/div[2]/div[{footnote_num}]',
        ).send_keys(footnote_content)

        driver.find_element(By.CLASS_NAME, "article-body").click()



# Generated by Django 3.2.13 on 2022-07-22 17:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0006_auto_20220715_1350"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="abstract",
            field=models.TextField(default=""),
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-15 08:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0004_publication_title"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="authors",
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name="publication",
            name="keywords",
            field=models.JSONField(default=list),
        ),
    ]



# Generated by Django 3.2.13 on 2022-05-27 17:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("document", "0018_fidus_3_4"),
    ]

    operations = [
        migrations.CreateModel(
            name="Publication",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("unsubmitted", "Unsubmitted"),
                            ("submitted", "Submitted"),
                            ("published", "Published"),
                            ("resubmitted", "Resubmitted"),
                        ],
                        default="unsubmitted",
                        max_length=11,
                    ),
                ),
                ("message_to_editor", models.TextField(default="")),
                (
                    "document",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="document.document",
                    ),
                ),
                (
                    "submitter",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Editor",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.1.4 on 2021-05-24 17:07

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0005_delete_teammember"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserInvite",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "key",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, unique=True
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        max_length=254, verbose_name="email address"
                    ),
                ),
                ("username", models.CharField(max_length=150)),
                (
                    "by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="invites_by",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from django.conf.urls import include
from django.urls import re_path
from django.conf import settings
from django.views.generic import RedirectView
from . import views

urlpatterns = [
    re_path("^save/$", views.save_profile, name="save_profile"),
    re_path("^avatar/delete/$", views.delete_avatar, name="delete_avatar"),
    re_path("^avatar/upload/$", views.upload_avatar, name="upload_avatar"),
    re_path(
        "^passwordchange/$", views.password_change, name="password_change"
    ),
    re_path("^email/add/$", views.add_email, name="add_email"),
    re_path("^email/delete/$", views.delete_email, name="delete_email"),
    re_path("^email/primary/$", views.primary_email, name="primary_email"),
    re_path(
        "^socialaccountdelete/$",
        views.delete_socialaccount,
        name="delete_socialaccount",
    ),
    # Delete a user
    re_path("^delete/$", views.delete_user, name="delete_user"),
    # Show contacts
    re_path("^contacts/list/$", views.list_contacts, name="list_contacts"),
    re_path(
        "^contacts/delete/$", views.delete_contacts, name="delete_contacts"
    ),
    re_path("^invite/$", views.invite, name="invite"),
    re_path("^invites/add/$", views.invites_add, name="invites_add"),
    re_path("^invites/accept/$", views.invites_accept, name="invites_accept"),
    re_path(
        "^invites/decline/$", views.invites_decline, name="invites_decline"
    ),
    # User avatar handling
    re_path("^avatar/", include("avatar.urls")),
    re_path(
        "^get_confirmkey_data/$",
        views.get_confirmkey_data,
        name="get_confirmkey_data",
    ),
    re_path(r"^signup/$", views.signup, name="account_signup"),
    # Authentication handling
    re_path("", include("allauth.urls")),
]

if not settings.PASSWORD_LOGIN:
    urlpatterns.insert(0, re_path("^login/$", RedirectView.as_view(url="/")))



import json

from django.http import JsonResponse
from django.contrib.auth.decorators import login_required
from django.db.models import Q
from django.views.decorators.http import require_POST

from base.decorators import ajax_required
from document.models import DocumentTemplate, FW_DOCUMENT_VERSION
from document.helpers.serializers import PythonWithURLSerializer
from style.models import DocumentStyle, DocumentStyleFile, ExportTemplate


@login_required
@ajax_required
@require_POST
def get_template(request):
    id = int(request.POST["id"])
    if id == 0:
        doc_template = DocumentTemplate()
        doc_template.user = request.user
        doc_template.save()
        status = 201
    else:
        doc_template = (
            DocumentTemplate.objects.filter(id=id)
            .filter(Q(user=request.user) | Q(user=None))
            .first()
        )
        status = 200
    if doc_template is None:
        return JsonResponse({}, status=405)
    serializer = PythonWithURLSerializer()
    export_templates = serializer.serialize(
        doc_template.exporttemplate_set.all()
    )
    document_styles = serializer.serialize(
        doc_template.documentstyle_set.all(),
        use_natural_foreign_keys=True,
        fields=["title", "slug", "contents", "documentstylefile_set"],
    )
    response = {
        "id": doc_template.id,
        "title": doc_template.title,
        "content": doc_template.content,
        "doc_version": doc_template.doc_version,
        "export_templates": export_templates,
        "document_styles": document_styles,
    }
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def list(request):
    response = {}
    status = 200
    doc_templates = DocumentTemplate.objects.filter(
        Q(user=request.user) | Q(user=None)
    )
    date_format = "%Y-%m-%d"
    response["document_templates"] = [
        {
            "id": obj.id,
            "title": obj.title,
            "is_owner": (obj.user is not None),
            "added": obj.added.strftime(date_format),
            "updated": obj.updated.strftime(date_format),
        }
        for obj in doc_templates
    ]
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def save(request):
    id = request.POST["id"]
    doc_template = DocumentTemplate.objects.filter(
        id=id, user=request.user
    ).first()
    if doc_template is None:
        return JsonResponse({}, status=405)
    response = {}
    status = 200
    doc_template.content = json.loads(request.POST["value"])
    doc_template.title = request.POST["title"]
    doc_template.import_id = request.POST["import_id"]
    doc_template.save()
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def create(request):
    response = {}
    title = request.POST.get("title")
    content = json.loads(request.POST.get("content"))
    import_id = request.POST.get("import_id")
    document_styles = json.loads(request.POST.get("document_styles"))
    export_templates = json.loads(request.POST.get("export_templates"))
    counter = 0
    base_title = title
    while DocumentTemplate.objects.filter(
        Q(title=title), Q(user=request.user) | Q(user=None)
    ).first():
        counter += 1
        title = f"{base_title} {counter}"
    template = DocumentTemplate.objects.create(
        title=title,
        content=content,
        doc_version=FW_DOCUMENT_VERSION,
        import_id=import_id,
        user=request.user,
    )
    response["id"] = template.id
    response["title"] = template.title
    date_format = "%Y-%m-%d"
    response["added"] = template.added.strftime(date_format)
    response["updated"] = template.updated.strftime(date_format)
    files = request.FILES.getlist("files[]")
    for style in document_styles:
        doc_style = DocumentStyle.objects.create(
            title=style["title"],
            slug=style["slug"],
            contents=style["contents"],
            document_template=template,
        )
        for filepath in style["files"]:
            filename = filepath.split("/").pop()
            file = next((x for x in files if x.name == filename), None)
            if file:
                DocumentStyleFile.objects.create(file=file, style=doc_style)
    for e_template in export_templates:
        filename = e_template["file"].split("/").pop()
        file = next((x for x in files if x.name == filename), None)
        if file:
            ExportTemplate.objects.create(
                document_template=template,
                template_file=file,
                file_type=e_template["file_type"],
            )
    return JsonResponse(response, status=201)


@login_required
@ajax_required
@require_POST
def copy(request):
    id = request.POST["id"]
    title = request.POST["title"]
    doc_template = DocumentTemplate.objects.filter(
        Q(id=id), Q(user=request.user) | Q(user=None)
    ).first()
    if doc_template is None:
        return JsonResponse({}, status=405)
    counter = 0
    base_title = title
    while DocumentTemplate.objects.filter(
        Q(title=title), Q(user=request.user) | Q(user=None)
    ).first():
        counter += 1
        title = f"{base_title} {counter}"
    response = {}
    status = 201
    document_styles = [style for style in doc_template.documentstyle_set.all()]
    export_templates = [
        template for template in doc_template.exporttemplate_set.all()
    ]
    doc_template.pk = None
    doc_template.title = title
    doc_template.user = request.user
    doc_template.save()
    for ds in document_styles:
        style_files = [file for file in ds.documentstylefile_set.all()]
        ds.pk = None
        ds.document_template = doc_template
        ds.save()
        for sf in style_files:
            sf.pk = None
            sf.style = ds
            sf.save()
    for et in export_templates:
        et.pk = None
        et.document_template = doc_template
        et.save()
    response["id"] = doc_template.id
    response["title"] = doc_template.title
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def delete(request):
    response = {}
    status = 405
    id = int(request.POST["id"])
    doc_template = DocumentTemplate.objects.filter(
        pk=id, user=request.user
    ).first()
    if doc_template:
        status = 200
        if doc_template.is_deletable():
            doc_template.delete()
            response["done"] = True
        else:
            response["done"] = False
    return JsonResponse(response, status=status)



from builtins import str
from builtins import object

from django.db import models
from django.db.utils import OperationalError, ProgrammingError
from django.core import checks
from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

# FW_DOCUMENT_VERSION:
# Also defined in frontend
# document/static/js/modules/schema/index.js

FW_DOCUMENT_VERSION = 3.4


class DocumentTemplate(models.Model):
    title = models.CharField(max_length=255, default="", blank=True)
    import_id = models.CharField(max_length=255, default="", blank=True)
    content = models.JSONField(default=dict)
    doc_version = models.DecimalField(
        max_digits=3, decimal_places=1, default=FW_DOCUMENT_VERSION
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.deletion.CASCADE,
    )
    added = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    auto_delete = True

    def __str__(self):
        return self.title

    def is_deletable(self):
        reverse_relations = [
            f
            for f in self._meta.model._meta.get_fields()
            if (f.one_to_many or f.one_to_one)
            and f.auto_created
            and not f.concrete
            and f.name not in ["documentstyle", "exporttemplate"]
        ]

        for r in reverse_relations:
            if r.remote_field.model.objects.filter(
                **{r.field.name: self}
            ).exists():
                return False
        return True

    @classmethod
    def check(cls, **kwargs):
        errors = super().check(**kwargs)
        errors.extend(cls._check_doc_versions(**kwargs))
        return errors

    @classmethod
    def _check_doc_versions(cls, **kwargs):
        try:
            if len(
                cls.objects.filter(doc_version__lt=str(FW_DOCUMENT_VERSION))
            ):
                return [
                    checks.Warning(
                        "Document templates need to be upgraded. Please "
                        "navigate to /admin/document/document/maintenance/ "
                        "with a browser as a superuser and upgrade all "
                        "document templates on this server.",
                        obj=cls,
                    )
                ]
            else:
                return []
        except (ProgrammingError, OperationalError):
            # Database has not yet been initialized, so don't throw any error.
            return []


class Document(models.Model):
    title = models.CharField(max_length=255, default="", blank=True)
    path = models.TextField(default="", blank=True)
    content = models.JSONField(default=dict)
    doc_version = models.DecimalField(
        max_digits=3, decimal_places=1, default=FW_DOCUMENT_VERSION
    )
    # The doc_version is the version of the data format in the content field.
    # We upgrade the content field in JavaScript and not migrations so that
    # the same code can be used for migrations and for importing old fidus
    # files that are being uploaded. This field is only used for upgrading data
    # and is therefore not handed to the editor or document overview page.
    version = models.PositiveIntegerField(default=0)
    diffs = models.JSONField(default=list, blank=True)
    # The last few diffs that were received and approved. The number of stored
    # diffs should always be equivalent to or more than all the diffs since the
    # last full save of the document.
    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name="owner",
        on_delete=models.deletion.CASCADE,
    )
    added = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    comments = models.JSONField(default=dict, blank=True)
    bibliography = models.JSONField(default=dict, blank=True)
    # Whether or not document is listed on document overview list page.
    # True by default and for all normal documents. Can be set to False when
    # documents are added in plugins that list these documents somewhere else.
    listed = models.BooleanField(default=True)
    template = models.ForeignKey(
        DocumentTemplate, on_delete=models.deletion.CASCADE
    )

    def __str__(self):
        if len(self.title) > 0:
            return "%(title)s (%(id)s)" % {"title": self.title, "id": self.id}
        else:
            return str(self.id)

    class Meta(object):
        ordering = ["-id"]

    def clean(self, *args, **kwargs):
        if self.comments is None:
            self.comments = "{}"
        if self.bibliography is None:
            self.bibliography = "{}"

    def save(self, *args, **kwargs):
        self.clean()
        return super().save(*args, **kwargs)

    def get_absolute_url(self):
        return "/document/%i/" % self.id

    def is_deletable(self):
        reverse_relations = [
            f
            for f in self._meta.model._meta.get_fields()
            if (f.one_to_many or f.one_to_one)
            and f.auto_created
            and not f.concrete
            and f.name
            not in [
                "accessright",
                "accessrightinvite",
                "documentrevision",
                "documentimage",
            ]
        ]

        for r in reverse_relations:
            if r.remote_field.model.objects.filter(
                **{r.field.name: self}
            ).exists():
                return False
        return True

    @classmethod
    def check(cls, **kwargs):
        errors = super().check(**kwargs)
        errors.extend(cls._check_doc_versions(**kwargs))
        return errors

    @classmethod
    def _check_doc_versions(cls, **kwargs):
        try:
            if len(
                cls.objects.filter(doc_version__lt=str(FW_DOCUMENT_VERSION))
            ):
                return [
                    checks.Warning(
                        "Documents need to be upgraded. Please navigate to "
                        "/admin/document/document/maintenance/ with a browser "
                        "as a superuser and upgrade all documents on this "
                        "server.",
                        obj=cls,
                    )
                ]
            else:
                return []
        except (ProgrammingError, OperationalError):
            # Database has not yet been initialized, so don't throw any error.
            return []


RIGHTS_CHOICES = (
    ("write", "Writer"),
    # Can write content and can read+write comments.
    # Can chat with collaborators.
    # Has read access to revisions.
    ("write-tracked", "Write with tracked changes"),
    # Can write tracked content and can read/write comments.
    # Cannot turn off tracked changes.
    # Can chat with collaborators.
    # Has read access to revisions.
    ("comment", "Commentator"),
    # Can read content and can read+write comments.
    # Can chat with collaborators.
    # Has read access to revisions.
    ("review-tracked", "Reviewer who can write with tracked changes"),
    # Can write tracked content and can read/write his own comments.
    # Cannot turn off tracked changes.
    # Cannot chat with collaborators.
    # Has no access to revisions.
    ("review", "Reviewer"),
    # Can read the content and can read/write his own comments.
    # Comments by users with this access right only show the user's
    # numeric ID, not their username.
    # Cannot chat with collaborators nor see that they are connected.
    # Has no access to revisions.
    ("read", "Reader"),
    # Can read content, including comments
    # Can chat with collaborators.
    # Has read access to revisions.
    ("read-without-comments", "Reader without comment access"),
    # Can read content, but not the comments.
    # Cannot chat with collaborators.
    # Has no access to revisions.
)

# Editor and Reviewer can only comment and not edit document
COMMENT_ONLY = ("review", "comment")

CAN_UPDATE_DOCUMENT = [
    "write",
    "write-tracked",
    "review",
    "review-tracked",
    "comment",
]

# Whether the collaborator is allowed to know about other collaborators
# and communicate with them.
CAN_COMMUNICATE = ["read", "write", "comment", "write-tracked"]


class AccessRight(models.Model):
    document = models.ForeignKey(Document, on_delete=models.deletion.CASCADE)
    path = models.TextField(default="", blank=True)
    holder_choices = models.Q(app_label="user", model="user") | models.Q(
        app_label="user", model="userinvite"
    )
    holder_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, limit_choices_to=holder_choices
    )
    holder_id = models.PositiveIntegerField()
    holder_obj = GenericForeignKey("holder_type", "holder_id")
    rights = models.CharField(
        max_length=21, choices=RIGHTS_CHOICES, blank=False
    )

    class Meta(object):
        unique_together = (("document", "holder_type", "holder_id"),)

    def __str__(self):
        return "%(name)s %(rights)s on %(doc_id)d" % {
            "name": self.holder_obj.readable_name,
            "rights": self.rights,
            "doc_id": self.document.id,
        }


def revision_filename(instance, filename):
    return "document-revisions/{id}.fidus".format(id=instance.pk)


class DocumentRevision(models.Model):
    document = models.ForeignKey(Document, on_delete=models.deletion.CASCADE)
    doc_version = models.DecimalField(
        max_digits=3, decimal_places=1, default=FW_DOCUMENT_VERSION
    )
    note = models.CharField(max_length=255, default="", blank=True)
    date = models.DateTimeField(auto_now=True)
    file_object = models.FileField(upload_to=revision_filename)
    file_name = models.CharField(max_length=255, default="", blank=True)

    def save(self, *args, **kwargs):
        if self.pk is None:
            # We remove the file_object the first time so that we can use the
            # pk as the name of the saved revision file.
            file_object = self.file_object
            self.file_object = None
            super().save(*args, **kwargs)
            self.file_object = file_object
            kwargs.pop("force_insert", None)
        super(DocumentRevision, self).save(*args, **kwargs)

    def __str__(self):
        if len(self.note) > 0:
            return "%(note)s (%(id)s) of %(doc_id)s" % {
                "note": self.note,
                "id": self.id,
                "doc_id": self.document.id,
            }
        else:
            return "%(id)s of %(doc_id)s" % {
                "id": self.id,
                "doc_id": self.document.id,
            }

    @classmethod
    def check(cls, **kwargs):
        errors = super().check(**kwargs)
        errors.extend(cls._check_doc_versions(**kwargs))
        return errors

    @classmethod
    def _check_doc_versions(cls, **kwargs):
        try:
            if len(
                cls.objects.filter(doc_version__lt=str(FW_DOCUMENT_VERSION))
            ):
                return [
                    checks.Warning(
                        "Document revisions need to be upgraded. Please "
                        "navigate to /admin/document/document/maintenance/ "
                        "with a browser as a superuser and upgrade all "
                        "document revisions on this server.",
                        obj=cls,
                    )
                ]
            else:
                return []
        except (ProgrammingError, OperationalError):
            # Database has not yet been initialized, so don't throw any error.
            return []



import os

from base.management import BaseCommand
from django.core.management.utils import get_random_secret_key
from django.conf import settings


class Command(BaseCommand):
    help = (
        "Creates a new Fidus Writer project in the current directory or "
        "optionally in the given directory."
    )

    def add_arguments(self, parser):
        parser.add_argument(
            "directory", nargs="?", help="Optional destination directory"
        )

    def handle(self, *args, **options):
        TARGET_PATH = options.pop("directory")
        SECRET_KEY = get_random_secret_key()
        if not TARGET_PATH:
            TARGET_PATH = settings.PROJECT_PATH
        if not os.path.exists(TARGET_PATH):
            os.makedirs(TARGET_PATH)
        SRC_PATH = settings.SRC_PATH
        with open(os.path.join(SRC_PATH, "configuration.py-default")) as file:
            CONFIGURATION = file.read()
        CONFIGURATION += "\n# Don't share the SECRET_KEY with anyone."
        CONFIGURATION += "\nSECRET_KEY = '{}'".format(SECRET_KEY)
        with open(os.path.join(TARGET_PATH, "configuration.py"), "w") as file:
            file.write(CONFIGURATION)



# Generated by Django 3.1.4 on 2021-04-06 04:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("document", "0012_auto_20210405_1821"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="accessright",
            name="user",
        ),
        migrations.AlterField(
            model_name="accessright",
            name="holder_id",
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name="accessright",
            name="holder_type",
            field=models.ForeignKey(
                limit_choices_to=models.Q(
                    ("app_label", "user"), ("model", "user")
                ),
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.contenttype",
            ),
        ),
    ]



import os

PORT = 8000

# If you want to show debug messages, set DEBUG to True.
DEBUG = True
SOURCE_MAPS = False

# This determines whether the server is used for testing and will let the
# users know upon signup know that their documents may disappear.
TEST_SERVER = True
# This is the contact email that will be shown in various places all over
# the site.
CONTACT_EMAIL = "mail@email.com"
# If websockets is running on a non-standard port, add it here:
WS_PORT = False
# Interval between document saves
DOC_SAVE_INTERVAL = 1

ADMINS = (("Your Name", "your_email@example.com"),)

MANAGERS = ADMINS

# The top path of the project. Depending on whether ./manage.py is executed or
# the fiduswriter pip package, it is either the dir that contains manage.py or
# the cwd.
PROJECT_PATH = os.environ.get("PROJECT_PATH")
# SRC_PATH is the root path of the FW sources.
SRC_PATH = os.environ.get("SRC_PATH")

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(PROJECT_PATH, "fiduswriter.sql"),
        "CONN_MAX_AGE": None,
    }
}

# Will let any user not delete more than 5000 bibliography entries
# simultaneously
DATA_UPLOAD_MAX_NUMBER_FIELDS = 5000

# Whether anyone surfing to the site can open an account with a login/password.
REGISTRATION_OPEN = True

# Whether user's can login using passwords (if not, they will only be able to
# sign in using social accounts).
PASSWORD_LOGIN = True

# Whether anyone surfing to the site can open an account or login with a
# socialaccount.
SOCIALACCOUNT_OPEN = True
SOCIALACCOUNT_STORE_TOKENS = True
SOCIALACCOUNT_EMAIL_VERIFICATION = "none"

# This determines whether there is a star labeled "Free" on the login page
IS_FREE = True

# Send emails to console.
EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"

# Or send emails using an SMTP server
# EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
# EMAIL_HOST = 'localhost'
# EMAIL_HOST_USER = ''
# EMAIL_HOST_PASSWORD = ''
# EMAIL_PORT = 25
# EMAIL_SUBJECT_PREFIX = '[Fidus Writer]'


# Local time zone for this installation. Keep UTC here, the frontend will
# translate this to the correct local time.
TIME_ZONE = "UTC"

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = "en-us"

SITE_ID = 1

# If you set this to False, the server will make some optimizations so as not
# to load the internationalization machinery. Fidus Writer makes use of
# internationalization, so you should probably keep this on.

USE_I18N = True

LOCALE_PATHS = (os.path.join(SRC_PATH, "locale"),)

# A list of allowed hostnames of this Fidus Writer installation
ALLOWED_HOSTS = [
    "localhost",
]

# If you set this to False, the server will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, the server will not use timezone-aware datetimes.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# The default is the media folder in the directory above this file.
MEDIA_ROOT = os.path.join(PROJECT_PATH, "media/")

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
MEDIA_URL = "/media/"

# The maximum size of user uploaded images in bytes. If you use NGINX, note
# that also it needs to support at least this size.
MEDIA_MAX_SIZE = False

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(PROJECT_PATH, "static-collected/")

# URL prefix for static files.
# Example: "https://media.lawrence.com/static/"
STATIC_URL = "/static/"

# Additional locations of static files
STATICFILES_DIRS = (
    os.path.join(PROJECT_PATH, "static-transpile"),
    os.path.join(PROJECT_PATH, "static-libs"),
)

LOGIN_URL = "/"
SOCIALACCOUNT_LOGIN_ON_GET = True

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    # 'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody. Change the default string.
SECRET_KEY = "2ouq2zgw5y-@w+t6!#zf#-z1inigg7$lg3p%8e3kkob1bf$#p4"


# These middleware classes is what Fidus Writer needs in its standard setup.
# You only need to change this in very advanced setups.
BASE_MIDDLEWARE = [
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.locale.LocaleMiddleware",
]

MIDDLEWARE = []


# The location of the top urls.py file inside the base folder.
# You only need to change this in very advanced setups.
ROOT_URLCONF = "base.root_urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(PROJECT_PATH, "templates"),
            # Put strings here, like "/home/html/django_templates".
            # You only need to change this in very advanced setups.
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.request",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]


# The following are the apps needed by Fidus Writer.

BASE_INSTALLED_APPS = [
    "npm_mjs",
    "base",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django.contrib.admindocs",
    "django.contrib.flatpages",
    "django_js_error_hook",
    "loginas",
    "fixturemedia",
    "browser_check",
    "menu",
    "document",
    "bibliography",
    "usermedia",
    "user",
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "avatar",
    "feedback",
    "style",
]

# These are additional apps to be overriden by configuration.py
INSTALLED_APPS = []

# These are apps that are removed from core apps overriden by configuration.py
REMOVED_APPS = []


AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend",
)

TEST_RUNNER = "django.test.runner.DiscoverRunner"
TESTING = False  # Overriden by test runner.


# Define available languages
# You only need to change this in very advanced setups.
def gettext(s):
    return s


LANGUAGES = (
    ("en", gettext("English")),
    ("bg", gettext("Bulgarian")),
    ("de", gettext("German")),
    ("fr", gettext("French")),
    ("it", gettext("Italian")),
    ("es", gettext("Spanish")),
    ("pt-br", gettext("Portuguese (Brazil)")),
)

LOGIN_REDIRECT_URL = "/"


# Allow users with login_as permission to log in as different user
def can_login_as(request, target_user):
    return request.user.has_perm("user.can_login_as")


CAN_LOGIN_AS = can_login_as

ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_EMAIL_VERIFICATION = "mandatory"
ACCOUNT_DEFAULT_HTTP_PROTOCOL = "http"

# allow login either with email or username
ACCOUNT_AUTHENTICATION_METHOD = "username_email"
ACCOUNT_ADAPTER = "user.adapter.AccountAdapter"

AUTH_PROFILE_MODULE = "account.Profile"
AUTH_USER_MODEL = "user.User"

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {
        "require_debug_false": {"()": "django.utils.log.RequireDebugFalse"}
    },
    "formatters": {
        "verbose": {
            "format": (
                "%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d "
                "%(message)s"
            )
        },
        "simple": {"format": "\033[22;32m%(levelname)s\033[0;0m %(message)s"},
    },
    "handlers": {
        "mail_admins": {
            "level": "ERROR",
            "filters": ["require_debug_false"],
            "class": "django.utils.log.AdminEmailHandler",
        },
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "simple",
        },
        "null": {
            "class": "logging.NullHandler",
        },
    },
    "loggers": {
        "django.request": {
            "handlers": ["mail_admins"],
            "level": "ERROR",
            "propagate": True,
        },
        "django.security.DisallowedHost": {
            "handlers": ["null"],
            "propagate": False,
        },
        "javascript_error": {
            "handlers": ["mail_admins", "console"],
            "level": "ERROR",
            "propagate": True,
        },
        "tornado.access": {
            "handlers": ["null"],
            "propagate": False,
        },
    },
}

# Global variables for avatar
# You only need to change this in very advanced setups.
AVATAR_GRAVATAR_BACKUP = False
AVATAR_THUMB_FORMAT = "PNG"
AVATAR_DEFAULT_URL = "img/default_avatar.png"
AVATAR_DEFAULT_SIZE = 80
AVATAR_MAX_AVATARS_PER_USER = 1
AVATAR_CACHE_ENABLED = True


WEBSOCKET_PING_INTERVAL = 55

ADMIN_SITE_TITLE = gettext("Fidus Writer Admin")
ADMIN_SITE_HEADER = gettext("Fidus Writer Administration Site")
ADMIN_INDEX_TITLE = gettext("Welcome to the Fidus Writer Administration Site")

# The below allow login via JavaScript
SESSION_COOKIE_HTTPONLY = False
CSRF_COOKIE_HTTPONLY = False

# To make npm-mjs enable the webpack offline plugin
WEBPACK_CONFIG_TEMPLATE = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "webpack.config.template.js"
)

# JS error logging
JAVASCRIPT_ERROR_USERAGENT_BLACKLIST = ["googlebot", "bingbot", "cutycapt"]
JAVASCRIPT_ERROR_BLACKLIST = [
    "script error",
    "the operation is insecure",
    "parse error",
    "unhandledrejection, {}",
    "An unknown error occurred",
]

# The page to show while transpilation takes place.
SETUP_PAGE_PATH = os.path.join(SRC_PATH, "base/setup_page/")

# Whether to use the old JSON patch method rather than prosemirror-py.
# Tests have shown that the JSON patch method is faster for now. This setting
# will likely be removed once the prosemirror-py method has reached a similar
# speed.
JSONPATCH = False

# Whether to create a service worker on production sites
USE_SERVICE_WORKER = True


DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
SILENCED_SYSTEM_CHECKS = ["models.W042"]
